/**
 *
 * This file is part of Tulip (www.tulip-software.org)
 *
 * Authors: David Auber, Patrick Mary, Morgan Mathiaut
 * from the LaBRI Visualization Team
 * Last modification : 15/01/2010
 *
 * Tulip is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Tulip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 */

#ifndef GLBEZIERCURVE_H_
#define GLBEZIERCURVE_H_

#include <tulip/GlSimpleEntity.h>
#include <tulip/GlTools.h>
#include <tulip/GlShaderManager.h>

namespace tlp {

class TLP_GL_SCOPE GlBezierCurve : public GlSimpleEntity {

public:

	GlBezierCurve(const std::vector<Coord> &controlPoints,
				  const Color &beginColor, const Color &endColor,
				  const float &beginSize, const float &endSize,
				  const unsigned int nbCurvePoints = 100,
				  const std::string &texture = "");

	~GlBezierCurve();

	void draw(float lod,Camera *camera);

	void setTexture(const std::string &texture);

	void setOutlined(const bool outlined) {this->outlined = outlined;}

	void setOutlineColor(const Color &outlineColor) {this->outlineColor = outlineColor;}

	void translate(const Coord& mouvement);

	void getXML(xmlNodePtr rootNode);

	void setWithXML(xmlNodePtr rootNode);

protected:

	GlShaderProgram *bezierVertexShader;
	bool vboOk;
	std::string shaderProgramName;
	std::vector<Coord> controlPoints;
	GLfloat *controlPointsArray;
	int nbControlPoints;
	Color beginColor;
	Color endColor;
	float beginSize;
	float endSize;
	std::string texture;
	unsigned int nbCurvePoints;
	bool outlined;
	Color outlineColor;

private :

	void buildBezierVertexBuffers(const unsigned int nbCurvePoints);

	static std::map<unsigned int, GLfloat *> bezierVertexBuffersData;
	static std::map<unsigned int, std::vector<GLushort *> > bezierVertexBuffersIndices;
	static std::map<unsigned int, GLuint* > bezierVertexBuffersObject;
	static std::map<unsigned int, std::vector<GLfloat *> > bezierVertexBuffersDataNoVbo;
};

}
#endif /* GLBEZIERCURVE_H_ */
