/**
 *
 * This file is part of Tulip (www.tulip-software.org)
 *
 * Authors: David Auber, Patrick Mary, Morgan Mathiaut
 * from the LaBRI Visualization Team
 * Last modification : 15/01/2010
 *
 * Tulip is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Tulip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 */
#ifndef GL_HCVXHULL_H
#define GL_HCVXHULL_H

#include <tulip/Graph.h>

#include "tulip/GlLayer.h"

namespace tlp {

  class GlComposite;
  class ConvexHullItem;
  class GlConvexHull;

  /**
   * Build a hierarchy of convex hulls
   */
  class TLP_GL_SCOPE GlHierarchyConvexHulls {

  public:
    /**
     * Basic constructor
     */
    GlHierarchyConvexHulls(bool fatherHullsIsDeducedFromChilds = true) {
      deducedFromChilds = fatherHullsIsDeducedFromChilds;
    }
    virtual ~GlHierarchyConvexHulls() {}

    /**
     * Build the hierarchy for the given graph in GlLayer
     */
    virtual void compute(tlp::GlLayer*,tlp::Graph *);

    /**
     * Create the hierarchy of ConvexHullItem
     */
    ConvexHullItem *buildComposite(ConvexHullItem *convexHull, GlConvexHull *oldHull);
    void setToOld(ConvexHullItem *convexHull, GlConvexHull *oldHull);

    bool deducedFromChilds;
  };
}

#endif
